/*
oZone3D TechDemo - Soft Shadows.
(C)2006 by JeGX - http//www.oZone3D.Net
7x7 bluring filter.
*/

#define KERNEL_SIZE 49

uniform float texelSize;
uniform sampler2DShadow shadowMap;
uniform sampler2D colorMap;
uniform sampler2D normalMap;
uniform float kernel[KERNEL_SIZE];
uniform vec3 offset[KERNEL_SIZE];

varying vec4 projCoord;
varying vec3 lightVec, viewVec;




void main ()
{
	vec3 shadowUV = projCoord.xyz / projCoord.q;
	float shadowColor = 0.0;
	float kernel_sum = 0.0;
	int i = 0;

	
	for( i=0; i<KERNEL_SIZE; i++ )
	{
		kernel_sum += kernel[i];
		shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[i]).x * kernel[i];
	}	
	
	shadowColor /= kernel_sum;





	vec3 lVec = normalize(lightVec);
	vec3 vVec = normalize(viewVec);

	vec3 bump = texture2D(normalMap, gl_TexCoord[0].st).xyz * 2.0 - 1.0;


	float diffuse = max( dot(lVec, bump), 0.0 );
	float specular = pow(clamp(dot(reflect(-lVec, bump),vVec),
					 0.0, 1.0), 
					 gl_FrontMaterial.shininess );

	vec4 color_map = texture2D(colorMap, gl_TexCoord[0].st );

	//vec4 vAmbient = gl_LightSource[0].ambient * gl_FrontMaterial.ambient * color_map;
	vec4 vAmbient = vec4(0.0);
	vec4 vDiffuse = gl_LightSource[0].diffuse * gl_FrontMaterial.diffuse * diffuse * color_map;	
	vec4 vSpecular = gl_LightSource[0].specular * gl_FrontMaterial.specular * specular;	
	
	
	gl_FragColor = (vAmbient + (vDiffuse + vSpecular)) * shadowColor;	
}



