/*
oZone3D TechDemo - Soft Shadows.
(C)2006 by JeGX - http//www.oZone3D.Net
7x7 bluring filter.
*/

#define KERNEL_SIZE 49

uniform float texelSize;
uniform sampler2DShadow shadowMap;
uniform sampler2D colorMap;
uniform float kernel[KERNEL_SIZE];
uniform vec3 offset[KERNEL_SIZE];

varying vec4 projCoord;
varying vec3 normal, lightDir, eyeVec;


void main ()
{
	vec4 final_color = vec4(0.0);
	vec3 shadowUV = projCoord.xyz / projCoord.q;
	float shadowColor = 0.0;
	
	shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[25]).x / 4.0;
	shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[31]).x / 4.0;
	shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[42]).x / 4.0;
	shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[48]).x / 4.0;

	vec3 N = normalize(normal);
	vec3 L = normalize(lightDir);
	
	//float lambertTerm = max( dot(N,L), 0.0);
	float lambertTerm = dot(N,L);
	
	if( (shadowColor-1.0) * shadowColor * lambertTerm != 0.0 )
	{
		float kernel_sum = 0.0;
		int i = 0;
		shadowColor =0.0;
	
		for( i=0; i<KERNEL_SIZE; i++ )
		{
			kernel_sum += kernel[i];
			shadowColor += shadow2D(shadowMap, shadowUV.xyz + offset[i]).x * kernel[i];
		}	
		
		shadowColor /= kernel_sum;
	}
	
	vec4 color_map = texture2D(colorMap, gl_TexCoord[0].st );
	
	if(lambertTerm > 0.0)
	{
		final_color += gl_LightSource[0].diffuse * 
		               gl_FrontMaterial.diffuse * 
					   lambertTerm * color_map;	
		
		vec3 E = normalize(eyeVec);
		vec3 R = reflect(-L, N);
		float specular = pow( max(dot(R, E), 0.0), 
		                 gl_FrontMaterial.shininess );
		final_color += gl_LightSource[0].specular * 
		               gl_FrontMaterial.specular * 
					   specular * lambertTerm;	
	}

	gl_FragColor = final_color * shadowColor;		
}



